//
// Fit a two-value function with values c0,c1 to a sequence of numbers
// such that square deviation + penalty for change between
// values is minimized
//
// Use gradient optimization to compute c0 and c1 that maximize
// the partition function
//


#include <iostream>
#include <string>
#include <fstream>
#include <sstream>

#include "fitonoff.hh"

using  namespace std;

void
read_number_sequence(string &filename,numseq_t &numseq) {
    ifstream in(filename.c_str());
    
    double x;
    while(in >> x) {numseq.push_back(x);}
}

int
main(int argc, char **argv) {
    
    string filename="profile.dat"; // file that contains the sequence of numbers
    double delta_ab = 2; // penalty for a change a->b
    double delta_ba = 2; // penalty for a change b->a

    double beta=3;
    
    double c0=0.0;
    double c1=1.0;

    bool do_optimization=true; // whether to perform optimization

    if (argc>=2) {
	filename=argv[1];
    }

    if (argc>=3) {
	std::stringstream in(argv[2]);
	in >> delta_ab;
	delta_ba = delta_ab;
    }

    if (argc>=5) {
	std::stringstream in3(argv[3]);
	in3 >> c0;
	std::stringstream in4(argv[4]);
	in4 >> c1;
	
	do_optimization=false;
    }
    
    // read number sequence from file
    numseq_t numseq;
    read_number_sequence(filename, numseq);
    
    
    FitOnOff fns(numseq,delta_ab,delta_ba,beta);
    
    // compute viterbi for given c0,c1, and store viterbi path
    double viterbi_score;
    viterbi_score = fns.viterbi(c0,c1,true);
    
    //cout << "Score: "<<viterbi_score<<endl;

    //fns.print_tables();
    
    //cout.precision(2);

    //fns.write_viterbi_path_compact(cout);
    //fns.write_viterbi_path(cout,c0,c1);
    
    if (do_optimization) {
	//optimize a and b
	pair<double,double> opt = fns.optimize(c0,c1);
	
	//cout << "Optimal c0,c1: "<<opt.first<<","<<opt.second<<endl;
	
	viterbi_score = fns.viterbi(opt.first,opt.second,true);
	//cout << "Score: "<<viterbi_score<<endl;
	
	c0=opt.first;
	c1=opt.second;
    }
    
    //fns.write_viterbi_path_compact(cout);
    fns.write_viterbi_path(cout,c0,c1);
    
}

